"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventEmitter = void 0;
const Constants_1 = require("./Constants");
const Utils_1 = require("./Utils");
class EventEmitter {
    /**
     * Creates an instance of EventEmitter.
     * @param globalOption - Global options for the class.
     * @param globalOption.separator - The separator used across all listeners unless listener contains custom separator.
     */
    constructor(globalOption = { separator: Constants_1.defaultSeparator }) {
        this.eventNamespaces = {};
        this.eventFilters = [];
        this.wildCardNamespace = '';
        this.globalOption = Object.assign({}, globalOption);
    }
    /**
     * Sets global options for the EventEmitter.
     * @param options - Global options, such as the separator.
     */
    setGlobalOptions(options) {
        this.globalOption = Object.assign({}, options);
    }
    /**
     * Adds a listener for the specified event, optionally applying filters, throttling, debouncing, and setting priority.
     * @param event - The event name, possibly with a namespace.
     * @param listener - The function to be called when the event is emitted.
     * @param options - An optional object containing properties like `filter`, `throttle`, `debounce`, and `priority`.
     * @param options.filter - A filter function to determine whether to emit the event.
     * @param options.throttle - The time delay (in milliseconds) for throttling the listener's execution.
     * @param options.debounce - The time delay (in milliseconds) for debouncing the listener's execution.
     * @param options.priority - The priority of the listener, higher values execute first (default is 0).
     * @param options.separator - Separator used for parsing the event (if applicable, default is '.').
     */
    on(event, listener, option = {}) {
        const { filter, throttle, debounce, priority, separator } = option;
        const usedSeparator = (0, Utils_1.getPrioritizedValue)(this.globalOption.separator, separator);
        const eventInfo = { separator: usedSeparator, event };
        const [namespace, eventName] = (0, Utils_1.parseEvent)(event, usedSeparator);
        const throttledListener = throttle !== undefined
            ? this.throttle(listener, throttle, eventName)
            : debounce !== undefined
                ? this.debounce(listener, debounce, eventName)
                : listener;
        const listenerObject = { listener: throttledListener, priority: priority !== null && priority !== void 0 ? priority : 0, eventInfo };
        if (!this.eventNamespaces[namespace]) {
            this.eventNamespaces[namespace] = {};
        }
        if (!this.eventNamespaces[namespace][eventName]) {
            this.eventNamespaces[namespace][eventName] = { listeners: [] };
        }
        (0, Utils_1.insertSorted)(this.eventNamespaces[namespace][eventName].listeners, listenerObject);
        this.eventNamespaces[namespace][eventName].throttled = throttle !== undefined;
        if (filter) {
            this.eventFilters.push(filter);
        }
    }
    /**
     * Removes a previously added listener for the specified event.
     * @param event - The event name, possibly with a namespace.
     * @param listener - The listener function to be removed.
     */
    off(event, listener) {
        const separator = (0, Utils_1.findEventInfo)(event, this.eventNamespaces);
        const [namespace, eventName] = (0, Utils_1.parseEvent)(event, separator);
        const namespaceObject = this.eventNamespaces[namespace];
        if (namespaceObject && namespaceObject[eventName]) {
            const listeners = namespaceObject[eventName].listeners;
            const index = listeners.findIndex(l => l.listener === listener);
            if (index !== -1) {
                listeners.splice(index, 1);
                if (!listeners.length) {
                    delete namespaceObject[eventName];
                }
                if ((0, Utils_1.isObjectEmpty)(namespaceObject)) {
                    delete this.eventNamespaces[namespace];
                }
            }
        }
    }
    /**
     * Emits the specified event, calling all associated listeners.
     * @param event - The event name, possibly with a namespace.
     * @param args - Additional arguments to be passed to the listeners.
     * @returns A promise that resolves when all listeners have been executed.
     */
    emit(event, ...args) {
        return __awaiter(this, void 0, void 0, function* () {
            const separator = (0, Utils_1.findEventInfo)(event, this.eventNamespaces);
            const [namespace, eventName] = (0, Utils_1.parseEvent)(event, separator);
            const shouldEmit = this.eventFilters.length === 0 || this.eventFilters.some(filter => filter(eventName, namespace));
            if (shouldEmit) {
                yield Promise.all([
                    this.executeSpecificListeners(this.wildCardNamespace, Constants_1.defaultWildCard, eventName, separator, args),
                    this.wildCardNamespace !== namespace &&
                        this.executeSpecificListeners(namespace, Constants_1.defaultWildCard, eventName, separator, args),
                    this.executeSpecificListeners(Constants_1.defaultWildCard, eventName, eventName, separator, args),
                    this.executeSpecificListeners(namespace, eventName, eventName, separator, args)
                ]);
            }
        });
    }
    /**
     * Executes specific listeners for the given namespace, event, and wildcard combination.
     * @param namespace - The namespace for which listeners should be executed.
     * @param checkEventName - The event name or wildcard to check for listeners.
     * @param eventName - The actual event name for emitting.
     * @param separator - Separator used for parsing the event (if applicable, default is '.').
     * @param args - Additional arguments to be passed to the listeners.
     * @returns A promise that resolves when all specific listeners have been executed.
     */
    executeSpecificListeners(namespace, checkEventName, eventName, separator, args) {
        var _a, _b, _c, _d, _e, _f;
        return __awaiter(this, void 0, void 0, function* () {
            const specificListeners = (_c = (_b = (_a = this.eventNamespaces[namespace]) === null || _a === void 0 ? void 0 : _a[checkEventName]) === null || _b === void 0 ? void 0 : _b.listeners) !== null && _c !== void 0 ? _c : [];
            const isThrottled = (_f = (_e = (_d = this.eventNamespaces[namespace]) === null || _d === void 0 ? void 0 : _d[checkEventName]) === null || _e === void 0 ? void 0 : _e.throttled) !== null && _f !== void 0 ? _f : false;
            for (const { listener } of specificListeners.filter(({ eventInfo }) => eventInfo.separator === separator)) {
                try {
                    if (isThrottled) {
                        yield listener(eventName, ...args);
                    }
                    else {
                        yield listener(eventName, ...args);
                    }
                }
                catch (error) {
                    this.handleListenerError(eventName, error);
                }
            }
        });
    }
    /**
     * Handles errors that occur during the execution of a listener.
     * @param eventName - The name of the event for which the listener encountered an error.
     * @param error - The error object representing the encountered error.
     */
    handleListenerError(eventName, error) {
        console.error(`Error in listener for event ${eventName}:`, error);
        console.error(error.stack);
    }
    /**
     * Creates a throttled version of the given listener function, ensuring it is called at most once within a specified time interval.
     * @param fn - The listener function to be throttled.
     * @param delay - The time delay (in milliseconds) between allowed invocations of the throttled function.
     * @param eventName - The name of the event associated with the listener.
     * @returns A throttled listener function.
     */
    throttle(fn, delay, eventName) {
        let lastCallTime = 0;
        return function (...args) {
            return __awaiter(this, void 0, void 0, function* () {
                const now = Date.now();
                if (now - lastCallTime >= delay) {
                    lastCallTime = now;
                    yield fn(eventName, ...args);
                }
            });
        };
    }
    /**
     * Creates a debounced version of the given listener function, ensuring it is called after a specified delay with no new invocations.
     * @param fn - The listener function to be debounced.
     * @param delay - The time delay (in milliseconds) before the debounced function is called after the last invocation.
     * @param eventName - The name of the event associated with the listener.
     * @returns A debounced listener function.
     */
    debounce(fn, delay, eventName) {
        let timeout;
        return function (...args) {
            return __awaiter(this, void 0, void 0, function* () {
                clearTimeout(timeout);
                timeout = setTimeout(() => __awaiter(this, void 0, void 0, function* () {
                    yield fn(eventName, ...args);
                }), delay);
            });
        };
    }
}
exports.EventEmitter = EventEmitter;
